# 1.1 Instalación

## Tres formas de instalar Go

Existen muchas formas de configurar el entorno de desarrollo de Go en tu computadora, y puedes escoger la que más te guste. Las tres formas más comunes son las siguientes:

- Paquetes oficiales de instalación
	- El equipo de Go provee packetes de instalación convenientes para Windows, Linux, Mac y otros sistemas operativos. Este es probablemente el método más fácil para iniciar.
- Instalarlo desde el código fuente
	- Es popular con los desarrolladores que están familiarizados con sistemas tipo Unix.
- Usando herramientas de terceros.
	- Existen muchas herramientas de terceros y manejadores de paquetes para instalar Go, como apt-get en Ubuntu y Homebrew en la Mac.

En el caso que desees instalar más de una versión de Go en la misma computadora, deberías revisar la herramienta llamada [GVM](https://github.com/moovweb/gvm). Es la mejor herramienta que he visto hasta el momento para cumplir esta tarea, de otra forma tendrás que lidiar con eso tu mismo.

## Instalando desde el código fuente

Debido a que algunas partes de Go están escritas en Plan 9 C y ensamblador AT&T, debes instalar un compilador de C antes de continuar.

Si tienes una Mac y ya tienes instalado Xcode, entonces ya cuentas con el compilador.

En sistemas tipo Unix, debes instalar gcc o un compilador similar. Por ejemplo, usando el manejador de paquetes apt-get (incluído con Ubuntu) puedes instalar los compiladores requeridos de la siguiente forma:

 `sudo apt-get install gcc libc6-dev`

En Windows, debes instalar MinGW antes para poder instalar gcc. No olvides configurar tus variables de entorno después de que la instalación se ha completado. (Si estás utilizando una versión de Windows de 64 bits, entonces debes instalar la versión de 64 bits de MinGW)

El equipo Go usa [Mercurial](http://mercurial.selenic.com/downloads/) para manejar su código fuente, por lo que debes instalar esta herramienta para poder bajar el código fuente de Go.

En este punto ejecuta los siguientes comandos para clonar el código fuente de Go y compilarlo. (Clonará el código fuente en tú directorio actual. Cambia tu ruta actual antes de continuar. Esto puede tomar algún tiempo.)

	hg clone -u release https://code.google.com/p/go
	cd go/src
	./all.bash

Una instalación exitosa finalizará con el mensaje "ALL TESTS PASSED."

En Windows puedes lograr lo mismo ejecutando `all.bat`.

Si estas usando Windows el paquete de instalación establecerá tus variables de entorno automáticamente. En sistemas tipo Unix necesitas establecer estas variables de la siguiente manera. (Si tu versión de Go es mayor que 1.0 no debes establecer $GOBIN ya que estará relacionada automáticamente a tu $GOROOT/bin del que hablaremos en la sección siguiente)

    export GOROOT=$HOME/go
    export GOBIN=$GOROOT/bin
    export PATH=$PATH:$GOROOT/bin

Si ves la siguiente información en tu pantalla, todo está listo.

![](images/1.1.mac.png?raw=true)

Imagen 1.1 Información después de instalar desde el código fuente

Una vez que veas la información de uso de Go significa que has instalado exitosamente Go en tu computadora. Si dice "no such command", verifica que tu variable de entorno $PATH contiene la ruta de instalación de Go.

## Usando los paquetes de instalación estándar

Go tiene paquetes de instalación de un solo click para todo sistema operativo soportado. Estos paquetes instalarán Go en `/usr/local/go` (`c:\Go` en Windows) por defecto. Por supuesto esto puede ser modificado, pero también necesitas cambiar todas las variables de entorno manualmente a cómo mostramos anteriormente.

### Cómo revisar si tu sistema operativo es de 32-bits ó 64-bits?

Nuestro siguiente paso depende en el tipo de tu sistema operativo, así que debemos revisar esto antes de bajar los paquetes de instalación estándar.

Si estás usando Windows, presiona `Win+R` y luego ejecuta la herramienta de comandos. Escribe el comando `systeminfo` y te mostrará información útil. Busca la línea que dice "system type" si ves "x64-based PC" eso significa que tu sistema operativo es de 64-bits, de otra forma es de 32-bits.

Si eres un usuario de Mac recomiendo fuertemente que bajes el paquete de 64-bits, debido a que Go ya no soporta procesadores puros de 32-bits en Mac OSX.

Los usuarios de Linux pueden escribir `uname -a` en la terminal para ver la información del sistema.

Un sistema operativo de 64-bits mostrará lo siguiente:

    <alguna descripción> x86_64 x86_64 x86_64 GNU/Linux
    // algunas versiones como Ubuntu 10.04 mostrarán de la siguiente forma
    x86_64 GNU/Linux

En cambio los sistemas operativos de 32-bits mostrarán:

    <alguna descripción> i686 i686 i386 GNU/Linux

### Mac

Ve a la [página de descarga](http://code.google.com/p/go/downloads/list), escoge `go1.0.3.darwin-386.pkg` para sistemas de 32-bits y `go1.0.3.darwin-amd64.pkg` para sistemas de 64-bits. Ve hasta el final presionando "next",`~/go/bin` será agregado al $PATH de tu sistema al finalizar la instalación. Ahora abre la terminal y escribe `go`. Debes ver el mismo resultado que se muestra en la imagen 1.1.

### Linux

Ve a la [página de descarga](http://code.google.com/p/go/downloads/list), escoge `go1.0.3.linux-386.tar.gz` para sistemas de 32-bits y `go1.0.3.linux-amd64.tar.gz` para sistemas de 64-bits. Supongamos que quieres instalar Go en la ruta `$GO_INSTALL_DIR`. Descomprime el archivo `tar.gz` a la ruta que escojas usando el comando `tar zxvf go1.0.3.linux-amd64.tar.gz -C $GO_INSTALL_DIR`. Luego configura tu $PATH con el siguiente comando: `export PATH=$PATH:$GO_INSTALL_DIR/go/bin`. Ahora abre la terminal y escribe `go`. Debes ver el mismo resultado que se muestra en la imagen 1.1.

### Windows

Ve a la [página de descarga](http://code.google.com/p/go/downloads/list), escoge `go1.0.3.windows-386.msi` para sistemas de 32-bits y `go1.0.3.windows-amd64.msi` para sistemas de 64-bits. Ve hasta el final presionando "next", `c:/go/bin` será agregado al `path`. Ahora abre la terminal y escribe `go`. Debes ver el mismo resultado que se muestra en la imagen 1.1.

## Usando herramientas de terceros

### GVM

GVM es una herramienta de control de múltiples versiones de Go desarrollado por terceros, parecida a rvm para ruby. Es bien fácil de utilizar. Instala gvm escribiendo los siguientes comandos en tu terminal:

    bash < <(curl -s https://raw.github.com/moovweb/gvm/master/binscripts/gvm-installer)

Luego instalamos Go usando los siguientes comandos:

    gvm install go1.0.3
    gvm use go1.0.3

Cuando el proceso ha finalizado estamos listos.

### apt-get

Ubuntu es la versión de escritorio más popular de Linux. Utiliza `apt-get` para manejar paquetes. Podemos instalar Go usando los siguientes comandos.

    sudo add-apt-repository ppa:gophers/go
    sudo apt-get update
    sudo apt-get install golang-stable

### Homebrew

Homebrew es una herramienta para manejar software comúnmente usada en la Mac. Simplemente escribe lo siguiente para instalar Go.

    brew install go

## Links

- [Directorio](preface.md)
- Sección anterior: [Configurando el entorno de Go](01.0.md)
- Sección siguiente: [$GOPATH y el ambiente de trabajo](01.2.md)
