# 7.6 文字列の処理
文字列は我々が通常Web開発においてよく使うものです。ユーザの入力やデータベースでのデータの読み取りなどを含め、我々はよく文字列に対して分割、連結、変換といった操作を行います。この節ではGoの標準ライブラリにあるstringsとstrconvという２つのパッケージの関数を使ってどのようにして素早く操作を行うかご説明します。
## 文字列の操作
以下の関数はstringsパッケージに入っています。ここでは普段よく使う関数をいくつかご紹介します。詳細はオフィシャルドキュメントをご参照ください。

- func Contains(s, substr string) bool

	文字列sにsubstrが含まれるか判断します。bool値を返します。
	
		fmt.Println(strings.Contains("seafood", "foo"))
		fmt.Println(strings.Contains("seafood", "bar"))
		fmt.Println(strings.Contains("seafood", ""))
		fmt.Println(strings.Contains("", ""))
		//Output:
		//true
		//false
		//true
		//true

- func Join(a []string, sep string) string

	文字列連結。slice aに対しsepで連結します。
	
		s := []string{"foo", "bar", "baz"}
		fmt.Println(strings.Join(s, ", "))
		//Output:foo, bar, baz		
			
- func Index(s, sep string) int 

	文字列sでsepが存在する位置です。インデックスを返します。見つからなければ-1を返します。
	
		fmt.Println(strings.Index("chicken", "ken"))
		fmt.Println(strings.Index("chicken", "dmr"))
		//Output:4
		//-1

- func Repeat(s string, count int) string

	s文字列をcount回リピートします。最後にリピートされた文字列を返します。
	
		fmt.Println("ba" + strings.Repeat("na", 2))
		//Output:banana

- func Replace(s, old, new string, n int) string

	s文字列において、old文字列をnew文字列に置換します。nは置換回数を表しています。0以下では全て置換します。
	
		fmt.Println(strings.Replace("oink oink oink", "k", "ky", 2))
		fmt.Println(strings.Replace("oink oink oink", "oink", "moo", -1))
		//Output:oinky oinky oink
		//moo moo moo

- func Split(s, sep string) []string

	sepによってs文字列を分割します。sliceを返します。
	
		fmt.Printf("%q\n", strings.Split("a,b,c", ","))
		fmt.Printf("%q\n", strings.Split("a man a plan a canal panama", "a "))
		fmt.Printf("%q\n", strings.Split(" xyz ", ""))
		fmt.Printf("%q\n", strings.Split("", "Bernardo O'Higgins"))
		//Output:["a" "b" "c"]
		//["" "man " "plan " "canal panama"]
		//[" " "x" "y" "z" " "]
		//[""]

- func Trim(s string, cutset string) string

	s文字列の先頭と末尾からcutsetで指定した文字列を除去する。
	
		fmt.Printf("[%q]", strings.Trim(" !!! Achtung !!! ", "! "))
		//Output:["Achtung"]

- func Fields(s string) []string

	s文字列の空白文字を除去し、空白に従って分割されたsliceを返します。
	
		fmt.Printf("Fields are: %q", strings.Fields("  foo bar  baz   "))
		//Output:Fields are: ["foo" "bar" "baz"]


## 文字列の変換
文字列を変換する関数はstrconvにあります。以下はそのうちよく使われるもののリストでしかありません：

- Append シリーズの関数は整数などを文字列に変換した後、現在のバイト列に追加します。

		package main
		
		import (
			"fmt"
			"strconv"
		)
		
		func main() {
			str := make([]byte, 0, 100)
			str = strconv.AppendInt(str, 4567, 10)
			str = strconv.AppendBool(str, false)
			str = strconv.AppendQuote(str, "abcdefg")
			str = strconv.AppendQuoteRune(str, '单')
			fmt.Println(string(str))
		}

- Format シリーズの関数は他の型を文字列に変換します。

		package main
	
		import (
			"fmt"
			"strconv"
		)
		
		func main() {
			a := strconv.FormatBool(false)
			b := strconv.FormatFloat(123.23, 'g', 12, 64)
			c := strconv.FormatInt(1234, 10)
			d := strconv.FormatUint(12345, 10)
			e := strconv.Itoa(1023)
			fmt.Println(a, b, c, d, e)
		}

- Parse シリーズの関数は文字列をその他の型に変換します。
		
		package main

		import (
			"fmt"
			"strconv"
		)
		func checkError(e error){
			if e != nil{
				fmt.Println(e)
			}
		}
		func main() {
			a, err := strconv.ParseBool("false")
			checkError(err)
			b, err := strconv.ParseFloat("123.23", 64)
			checkError(err)
			c, err := strconv.ParseInt("1234", 10, 64)
			checkError(err)
			d, err := strconv.ParseUint("12345", 10, 64)
			checkError(err)
			e, err := strconv.Atoi("1023")
			checkError(err)
			fmt.Println(a, b, c, d, e)
		}

	

## links
   * [目次](<preface.md>)
   * 前へ: [ファイルの操作](<07.5.md>)
   * 次へ: [まとめ](<07.7.md>)
